// -----------------------------------------------------------------------------
//
// Copyright (C) 2021 CERN & Newcastle University for the benefit of the
// BioDynaMo collaboration. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
//
// See the LICENSE file distributed with this work for details.
// See the NOTICE file distributed with this work for additional information
// regarding copyright ownership.
//
// -----------------------------------------------------------------------------

#ifndef CORE_DIFFUSION_EULER_GRID_H_
#define CORE_DIFFUSION_EULER_GRID_H_

#include "core/diffusion/diffusion_grid.h"

namespace bdm {

class EulerGrid : public DiffusionGrid {
 public:
  EulerGrid() {}
  EulerGrid(int substance_id, std::string substance_name, double dc, double mu,
            int resolution = 11, double binding_coefficient = 0, size_t isDepletedBy = 0, double source = 0)
      : DiffusionGrid(substance_id, substance_name, dc, mu, resolution, binding_coefficient, isDepletedBy, source) {
   }

  void DiffuseWithClosedEdge() override;

  void DiffuseWithOpenEdge() override;

 private:
  BDM_CLASS_DEF_OVERRIDE(EulerGrid, 1);
};

}  // namespace bdm

#endif  // CORE_DIFFUSION_EULER_GRID_H_
