// -----------------------------------------------------------------------------
//
// Copyright (C) The BioDynaMo Project.
// All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
//
// See the LICENSE file distributed with this work for details.
// See the NOTICE file distributed with this work for additional information
// regarding copyright ownership.
//
// -----------------------------------------------------------------------------

#include "myParam.h"
#include "core/util/cpptoml.h"

#define BDM_ASSIGN_PARAM_VALUE(value) BDM_ASSIGN_CONFIG_VALUE(value, #value)

namespace bdm {

void SimParam::AssignFromConfig(const std::shared_ptr<cpptoml::table>& config) {


  // Operations
  BDM_ASSIGN_PARAM_VALUE(nb_of_sim_steps_);
  BDM_ASSIGN_PARAM_VALUE(nb_of_subs_);  // Nb of substances whose conc. we want to track
  BDM_ASSIGN_PARAM_VALUE(nb_of_cell_types_);  // Nb of cell types whose number we want to track
  BDM_ASSIGN_PARAM_VALUE(sim_time_step);
  BDM_ASSIGN_PARAM_VALUE(measurement_conc_ops_freq_);
  BDM_ASSIGN_PARAM_VALUE(measurement_cells_ops_freq_);

  // Various
  BDM_ASSIGN_PARAM_VALUE(M1_MCP1_chemotactic_sensitivity_);
  BDM_ASSIGN_PARAM_VALUE(M1_MCP1_degradation);
  BDM_ASSIGN_PARAM_VALUE(ECM_saturation);
  BDM_ASSIGN_PARAM_VALUE(IL13_effect_M2_TGFb_secretion);
  BDM_ASSIGN_PARAM_VALUE(TGFb_ac_effect_MF_ECM_secretion);
  BDM_ASSIGN_PARAM_VALUE(random_movement_probability);
  BDM_ASSIGN_PARAM_VALUE(cell_speed);
  BDM_ASSIGN_PARAM_VALUE(fibroblast_migration_speed);
  BDM_ASSIGN_PARAM_VALUE(sub_res);
  BDM_ASSIGN_PARAM_VALUE(beta);
  BDM_ASSIGN_PARAM_VALUE(input_file_name);

  // Secretion rates
  BDM_ASSIGN_PARAM_VALUE(M2_PDGF_secretion);
  BDM_ASSIGN_PARAM_VALUE(M2_MMP_secretion);
  BDM_ASSIGN_PARAM_VALUE(M2_TIMP_secretion);
  BDM_ASSIGN_PARAM_VALUE(M2_IL13_secretion);
  BDM_ASSIGN_PARAM_VALUE(M2_TGFb_secretion);
  BDM_ASSIGN_PARAM_VALUE(M1_TNFa_secretion);
  BDM_ASSIGN_PARAM_VALUE(AEC2_TNFa_secretion);
  BDM_ASSIGN_PARAM_VALUE(AEC2_MCP1_secretion);
  BDM_ASSIGN_PARAM_VALUE(AEC2_FGF2_secretion);
  BDM_ASSIGN_PARAM_VALUE(AEC2_TGFb_secretion);
  BDM_ASSIGN_PARAM_VALUE(F_TGFb_secretion);
  BDM_ASSIGN_PARAM_VALUE(F_ECM_secretion);
  BDM_ASSIGN_PARAM_VALUE(MF_ECM_secretion);
  BDM_ASSIGN_PARAM_VALUE(L_IL13_secretion);

  // Dissociation constants
  BDM_ASSIGN_PARAM_VALUE(k_IL13);
  BDM_ASSIGN_PARAM_VALUE(k_TGFb_ac);
  BDM_ASSIGN_PARAM_VALUE(k_TGFb_in);
  BDM_ASSIGN_PARAM_VALUE(k_TGFb_in_AEC2_FGF2);
  BDM_ASSIGN_PARAM_VALUE(k_MCP1);
  BDM_ASSIGN_PARAM_VALUE(k_FGF2);
  BDM_ASSIGN_PARAM_VALUE(k_PDGF);
  BDM_ASSIGN_PARAM_VALUE(k_TNFa);

  // Proliferation rates
  BDM_ASSIGN_PARAM_VALUE(F_proliferation_IL13_TGFb);
  BDM_ASSIGN_PARAM_VALUE(F_proliferation_FGF2);
  BDM_ASSIGN_PARAM_VALUE(AEC2_proliferation);
  BDM_ASSIGN_PARAM_VALUE(F_base_proliferation);
  BDM_ASSIGN_PARAM_VALUE(M0_basic_prob_production);
  BDM_ASSIGN_PARAM_VALUE(F_production);

  // Differentiation rates
  BDM_ASSIGN_PARAM_VALUE(AEC2_AEC1_differentiation);
  BDM_ASSIGN_PARAM_VALUE(F_MF_TGFb_ac_differentiation);
  BDM_ASSIGN_PARAM_VALUE(F_MF_PDGF_differentiation);
  BDM_ASSIGN_PARAM_VALUE(M1_M2_differentiation);
  BDM_ASSIGN_PARAM_VALUE(M2_M1_TNFa_differentiation);
  BDM_ASSIGN_PARAM_VALUE(M0_M1_differentiation);
  BDM_ASSIGN_PARAM_VALUE(AEC2_EMT_ECM_differentiation);
  BDM_ASSIGN_PARAM_VALUE(AEC2_EMT_constant_rate);
  BDM_ASSIGN_PARAM_VALUE(AEC2_F);
  BDM_ASSIGN_PARAM_VALUE(AEC2_MF);

  // Apoptosis rates
  BDM_ASSIGN_PARAM_VALUE(AEC1_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(AEC2_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(active_AEC2_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(F_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(M2_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(M1_apoptosis);
  BDM_ASSIGN_PARAM_VALUE(MF_apoptosis);

  // Damage/Repair
  BDM_ASSIGN_PARAM_VALUE(TGFb_ac_death_AEC2);
  BDM_ASSIGN_PARAM_VALUE(ROS_death_AEC2);
  BDM_ASSIGN_PARAM_VALUE(hitProbability);
  BDM_ASSIGN_PARAM_VALUE(fibrocytesRepair_AEC2);
  BDM_ASSIGN_PARAM_VALUE(setInitialDamageAEC2);
  BDM_ASSIGN_PARAM_VALUE(activationProbability_AEC2);
  BDM_ASSIGN_PARAM_VALUE(damageType);
  BDM_ASSIGN_PARAM_VALUE(clusteredDamageSize);

  // Depletion rates
  BDM_ASSIGN_PARAM_VALUE(TIMP_MMP_binding);
  BDM_ASSIGN_PARAM_VALUE(MMP_TIMP_binding);
  BDM_ASSIGN_PARAM_VALUE(MMP_ECM_binding);

  // Substances initial concentrations
  BDM_ASSIGN_PARAM_VALUE(flat_conc);
  BDM_ASSIGN_PARAM_VALUE(ECM_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(MCP1_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(PDGF_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(MMP_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(TGFb_in_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(TGFb_ac_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(TIMP_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(TNFa_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(IL13_flat_conc);
  BDM_ASSIGN_PARAM_VALUE(FGF2_flat_conc);

  // Substances diffusion constants
  BDM_ASSIGN_PARAM_VALUE(PDGF_D);
  BDM_ASSIGN_PARAM_VALUE(MMP_D);
  BDM_ASSIGN_PARAM_VALUE(ECM_D);
  BDM_ASSIGN_PARAM_VALUE(MCP1_D);
  BDM_ASSIGN_PARAM_VALUE(TIMP_D);
  BDM_ASSIGN_PARAM_VALUE(TGFb_in_D);
  BDM_ASSIGN_PARAM_VALUE(TGFb_ac_D);
  BDM_ASSIGN_PARAM_VALUE(TNFa_D);
  BDM_ASSIGN_PARAM_VALUE(IL13_D);
  BDM_ASSIGN_PARAM_VALUE(FGF2_D);

  // Substances decay constants
  BDM_ASSIGN_PARAM_VALUE(PDGF_d);
  BDM_ASSIGN_PARAM_VALUE(MMP_d);
  BDM_ASSIGN_PARAM_VALUE(ECM_d);
  BDM_ASSIGN_PARAM_VALUE(MCP1_d);
  BDM_ASSIGN_PARAM_VALUE(TIMP_d);
  BDM_ASSIGN_PARAM_VALUE(TGFb_in_d);
  BDM_ASSIGN_PARAM_VALUE(TGFb_ac_d);
  BDM_ASSIGN_PARAM_VALUE(TNFa_d);
  BDM_ASSIGN_PARAM_VALUE(IL13_d);
  BDM_ASSIGN_PARAM_VALUE(FGF2_d);

  BDM_ASSIGN_PARAM_VALUE(n_of_cells);
}

}  // namespace bdm
