// -----------------------------------------------------------------------------
//
// Copyright (C) Nicolò Cogno.
// All Rights Reserved.
//
// -----------------------------------------------------------------------------

#ifndef OUTPUT_UTILS_H_
#define OUTPUT_UTILS_H_

#include <TAxis.h>
#include <TCanvas.h>
#include <TF1.h>
#include <fstream>
#include <string>
#include <vector>
#include "TROOT.h"
#include "TTree.h"
#include <TGraph.h>
#include <TLegend.h>
#include <TLine.h>
#include <TMultiGraph.h>
#include <TStyle.h>

namespace bdm {
void export_to_CSV(std::string fileName, std::vector<ConcData*> conc,
                   const std::string& outputDir);

void plot_conc(std::string plot_fileName, std::vector<ConcData*> conc,
               const std::string& outputDir);

void plot_conc_all_in_one(std::string plot_fileName,
                          std::vector<ConcData*> conc,
                          const std::string& outputDir);

void plot_cells(std::string plot_fileName, std::vector<std::vector<int>> cells,
                size_t time_steps, size_t cell_types, size_t op_freq,
                const std::string& outputDir);

void export_to_CSV_cells(std::string fileName,
                         std::vector<std::vector<int>> cells, size_t time_steps,
                         size_t cell_types, size_t op_freq,
                         const std::string& outputDir);

void export_to_root_cells(std::string fileName,
                          std::vector<std::vector<double>> cells,
                          size_t time_steps, size_t cell_types, size_t op_freq,
                          const std::string& outputDir, int nb_of_alveoli);

void export_to_root_concs(std::string fileName, std::vector<ConcData*>* conc,
                          const std::string& outputDir);

void export_to_root_local_concs(std::string fileName,
                                std::vector<std::vector<double>> local_concs,
                                size_t time_steps, size_t nb_of_subs,
                                size_t op_freq, const std::string& outputDir);

void export_to_root_ecm_map(std::string fileName,
                            std::vector<double> ecm_conc_array,
                            const std::string& outputDir);

void export_to_root_local_concs(std::string fileName,
                                std::vector<std::vector<double>> local_concs,
                                size_t time_steps, size_t nb_of_subs,
                                size_t op_freq, const std::string& outputDir);

}  // namespace bdm

#endif
